"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.planDeployment = exports.createDeploymentTargetNamePatternMatcher = void 0;
const util_1 = require("@takomo/util");
const ramda_1 = __importDefault(require("ramda"));
const confirm_1 = require("./confirm");
const createDeploymentTargetNamePatternMatcher = (pattern) => {
    const prefix = pattern.endsWith("%");
    const suffix = pattern.startsWith("%");
    if (prefix && suffix) {
        const part = pattern.slice(1, -1);
        return ({ name }) => name.includes(part);
    }
    if (prefix) {
        const part = pattern.slice(0, -1);
        return ({ name }) => name.startsWith(part);
    }
    if (suffix) {
        const part = pattern.slice(1);
        return ({ name }) => name.endsWith(part);
    }
    return ({ name }) => name === pattern;
};
exports.createDeploymentTargetNamePatternMatcher = createDeploymentTargetNamePatternMatcher;
/**
 * @hidden
 */
const planDeployment = async (holder) => {
    const { ctx, io, timer, input: { groups, targets, excludeTargets, labels, excludeLabels, configSetType, }, } = holder;
    if (groups.length > 0) {
        groups.forEach((groupPath) => {
            if (!ctx.hasDeploymentGroup(groupPath)) {
                throw new util_1.TakomoError(`Deployment group '${groupPath}' not found`);
            }
        });
    }
    const deploymentGroupsToLaunch = groups.length === 0
        ? ctx.rootDeploymentGroups
        : groups.reduce((collected, path) => {
            return [...collected, ctx.getDeploymentGroup(path)];
        }, new Array());
    const sortGroups = (a, b) => {
        const order = a.priority - b.priority;
        return order !== 0 ? order : a.name.localeCompare(b.name);
    };
    const groupsToLaunch = deploymentGroupsToLaunch
        .map((ou) => util_1.collectFromHierarchy(ou, (o) => o.children, {
        sortSiblings: sortGroups,
        filter: (o) => o.status === "active",
    }))
        .flat();
    const uniqueGroupsToLaunch = ramda_1.default.uniqBy(ramda_1.default.prop("path"), groupsToLaunch).filter((o) => o.status === "active");
    const hasConfigSets = (target) => {
        switch (configSetType) {
            case "standard":
                return target.configSets.length > 0;
            case "bootstrap":
                return target.bootstrapConfigSets.length > 0;
            default:
                throw new Error(`Unsupported config set type: ${configSetType}`);
        }
    };
    const targetNameMatchers = targets.map(exports.createDeploymentTargetNamePatternMatcher);
    const excludeTargetNameMatchers = excludeTargets.map(exports.createDeploymentTargetNamePatternMatcher);
    const targetNameMatches = (target) => (targetNameMatchers.length === 0 ||
        targetNameMatchers.some((m) => m(target))) &&
        (excludeTargetNameMatchers.length === 0 ||
            !excludeTargetNameMatchers.some((m) => m(target)));
    const labelMatches = (target) => (labels.length === 0 || target.labels.some((l) => labels.includes(l))) &&
        (excludeLabels.length === 0 ||
            !target.labels.some((l) => excludeLabels.includes(l)));
    const grs = uniqueGroupsToLaunch
        .map((ou) => {
        return {
            ...ou,
            targets: ou.targets.filter((a) => a.status === "active" &&
                hasConfigSets(a) &&
                targetNameMatches(a) &&
                labelMatches(a)),
        };
    })
        .filter((ou) => ou.targets.length > 0);
    const hasChanges = grs.length > 0;
    if (!hasChanges) {
        timer.stop();
        io.info("No targets to deploy");
        return {
            results: [],
            timer,
            success: true,
            status: "SKIPPED",
            message: "No targets to deploy",
        };
    }
    const selectedTargets = grs.map((g) => g.targets).flat();
    const plan = {
        configSetType,
        groups: grs,
        hasChanges,
    };
    const listener = io.createDeploymentTargetsListener(selectedTargets.length);
    return confirm_1.confirmOperation({
        ...holder,
        plan,
        listener,
    });
};
exports.planDeployment = planDeployment;
//# sourceMappingURL=plan.js.map